/**
 * Name:Daniel Greco
 * Filename: Client.java
 * Description: This file will handle the client in the game. This will connect to a host and interact accordingly.
 */ 
import java.io.*;
import java.net.*;
import java.util.Scanner;

public class Client {

    private String name, hostName; //holds the name of player, and the name of the host.
    private Socket socket; //the socket to connect between the host and client
    private InetAddress inet; //used to get the host address
    private int port; //the port to connect. Will always be 4444
    private Scanner scan; 
    private int game; //what game to play
    
    /**
     * private default constructor for class client. This will never be accessed or used.
     * 
     */
    private Client()
    { 
    }
    
    /**
     * Constructor for class Client.
     * @param name is a String that contains the name the client gave in the main menu
     * @param inet is an InetAddress object which contains the host address. Used for connecting to the host
     * @param port is an int that will tell us what port we'll be using to connect. Always 4444
     * @param game is an int telling us what game we'll be playing
     * 
     */
    public Client(String name, InetAddress inet, int port, int game)
    {
        this.name = name;
        this.inet = inet;
        this.port = port;
        this.scan = new Scanner(System.in);
        this.game = game;
    }
    /**
     * run will simply run the interaction between the host and the client
     * @return true if the client still wants to play another game. False will completely
     * quit the program
     */
    public boolean run() throws IOException {

        ObjectOutputStream out = null;
        ObjectInputStream in = null;
        
        try {
            //Step 2: Initiate a connection request to server's IP and port
            socket = new Socket(inet.getHostAddress(), port);
            out = new ObjectOutputStream(socket.getOutputStream());
            in = new ObjectInputStream(socket.getInputStream());
        } catch (UnknownHostException e) {
            System.out.println("The host you gave does not exist. Taking you back to the main menu");
            return true;
            
        } catch (IOException e) {
            System.out.println("I'm sorry that host isn't available right now.  Taking you back to the main menu");
            return true;
            
        }
        
        //set up reading from the system in.
        BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));

        //a message object to hold server messages, and a string to hold what we enter in.
        Message fromServer;
        String fromUser;
        
        //Greet the user
        switch(this.game)
        {
            case MainMenu.NUM_GUESS:
                System.out.println("Welcome to the number counter game!");
                break;
            case MainMenu.ROCHAMBEAU:
                System.out.println("Are you ready for some Rochambeau!?\n");
                break;
            default:
                System.out.println("Oops you shouldn't be here.");
                break;
        }
        
        //Step 5: Receive message from server
        try
        {
            while ((fromServer = (Message)in.readObject()) != null) {
                //Step 6: Pring message on screen
                System.out.println(fromServer);
                //try to get the host name. However on the first communication this might be a problem
                //so we'll use a catch block just in case.
                try
                {
                    hostName  = fromServer.getName();        
                }
                catch(Exception e)
                {
                    hostName = "host";
                }

                //if we're quitting let the user know and return to the main menu
                if (((Message)fromServer).isQuit())
                {
                   System.out.println("Connection to " +hostName + " terminated.");
                   //return but keep running the main menu
                   return true;
                }

                //if we won, see if the user wants to play again
                else if(((Message)fromServer).isWin() || ((Message)fromServer).isOver())
                {
                    System.out.println("Would you like to play again? Type Yes or No Please.\n");
                    
                }
                
                //makes the client see they should be typing.
                System.out.print(">");
                //Step 7: Read response from user
                fromUser= stdIn.readLine();
            
            if (fromUser != null) {
                if(fromServer.getMsgType() == Message.GUESS)
                {
                    while(!RochambeauProtocol.isValidChoice(fromUser))
                    {
                        System.out.println("You are giving wrong input, try again.\n");
                        System.out.print(">");
                        fromUser = stdIn.readLine();
                    }
                }
                    //System.out.println(fromUser);
                    //Step 8:Send message to server including username!!!
                    out.writeObject(new Message(name, fromUser, Message.GUESS));
            }
            }
        }
        //host disconnect
        catch(IOException e)
        {
            System.out.println("I'm fairly certain the host doesn't like you and left the game.\nI'm taking you back to the main menu.\nPress any enter to continue");
            scan.nextLine();
            return true;
        }
        catch(Exception e)
        {
            System.err.println("Error during while loop on client side" + e.getMessage());
        }

        out.close();
        in.close();
        stdIn.close();
        socket.close();
        //return but exiting the entire program
        return false;
    }
}