/**
 * Name:Daniel Greco
 * Filename: HostThread.java
 * Description: This file will handle the interaction between the host and the client.  
 */ 

import java.net.*;
import java.io.*;

public class HostThread extends Thread {
    private Socket socket = null;
    private String name; //holds the name of host
    private int game; //holds the game number. Right now it's only 1
    
    /**
     * Constructor for class HostThread. 
	 * @param socket is a socket object, that will be used when connecting to a client
     * @param name is the host's name that was obtained from the main menu
	 * @param game is the game code that will be used to host
     */
    public HostThread(Socket socket, String name, int game) 
	{
    	super("HostThread");
    	this.socket = socket;
    	this.name = name;
    	this.game = game;
    }
    
    /**
     * Default Constructor for class HostThread. Note that this is private so it cannot be
	 *	called from the outside.
     * 
     */
    private HostThread()
    {
        
    }
    
    /**
     * Run is the main function for this class. It will handle the interaction
	 * between the client and the host
     * 
     */
    public void run() {
        Protocol kkp = null;
	//the entire method is enclosed in a try block because if the client disconnects at any time
	//we want to catch that exception and handle it appropriately
    try 
	{
    		//get new object streams
            ObjectOutputStream out = new ObjectOutputStream(socket.getOutputStream());
            ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
    		
    		//hold incoming messages and a way to store outgoing messages
            Message inputLine;
            Message outputLine;
            
            switch(this.game)
            {
                case MainMenu.NUM_GUESS:
                    //start up the protocol for the game
                    kkp = new NumGuessProtocol(this.name);
                    break;
                case MainMenu.ROCHAMBEAU:
                    kkp = new RochambeauProtocol(this.name);
                    break;
                default:
                    System.err.println("Bad Game Code Given");
                    break;
            }
    		
    		//get the first input that will be sent to the client
            outputLine = kkp.processInput(null);
    		//send to the client
            out.writeObject(outputLine);
            
    		//run this during the entire client-host interaction
            while ((inputLine = (Message)in.readObject()) != null) 
    		{
        		//process the incoming message and get a new message to send back to the client
                outputLine = kkp.processInput(((Message)inputLine));
        		//send the message back to the client.
                out.writeObject(outputLine);
        		//if our client wants to quit, we'll take care of that here.
                if (outputLine.isQuit())
                {
                    System.out.println("Connection to " + ((Message)inputLine).getName() + " terminated\nWaiting for a new player");
                    break;
                }
            }
    		//close the sockets!
            out.close();
            in.close();
            socket.close();
    
        }
    	//catch any disconnects 
        catch(IOException e)
        {
            System.out.println("I'm fairly certain the player doesn't like the way you're hosting and left the game.\nNow we'll go through the process of finding a new person again");
        }
    	//catch anything else
        catch (Exception e) {
            System.err.println("This isn't good. " + e.getMessage());
        }
    }
}