/**
 * Name:Daniel Greco
 * Filename: mainmenu.java
 * Description: This file will handle the initial setup of the game and contains the main class.
 */ 
import java.io.IOException;
import java.net.*;
import java.util.Scanner;

public class MainMenu
{
    //game codes
    public static final int PORT = 4443; //this will be the base port. It will have the game code added to it so each port runs a different game
    public static final int NUM_GUESS = 1;    
    public static final int ROCHAMBEAU = 2;
    public static final int HOST = 1;
    public static final int JOIN = 2;
    public static final int NUMGAMES = 2;
    
    /**
     * Main will start by grabbing the user's name. From there the runMenu method will handle the rest.
     * This ensures we only ask the user for their name once, but allow them to jump between games as a client.
     * 
     */
    public static void main(String args[])
    {
        String name; //player name
        Scanner scan = new Scanner(System.in); //scan for system.in
        boolean stillRunning; //used to know if we should continue or not.
        
        System.out.println("Welcome to the Arcade of wonders! (Type quit at any time to exit)");
        System.out.println("What's your name? I forgot again");
        
        //retrieve the name
        //no error checking because the name can be anything.
        name = scan.next();
        if(isQuit(name))
        {
            return;
        }
        System.out.println(name + "  hm... That's a nice name");
        
        //run this loop until the user chooses to quit
        do
        {
            stillRunning = runMenu(name);
        }while(stillRunning);
        
    }
    
    
    /**
     * runMenu will handle the selecting of the game, and whether to host or join a game
     * @return returns true if it should continue to run, false otherwise
     */
    private static boolean runMenu(String name)
    {
        Scanner scan = new Scanner(System.in);
        int game;

        //Retrieve the game the user would like to play.
        System.out.println(name + ", what game would you like to play today?(Type quit to quit at anytime");
        System.out.println("1)Number Guesser");
        System.out.println("2)Rochambeau");
        
        //get the choice from the user
        String choice = scan.next();
        if(isQuit(choice))
        {
            //exit and quit
            return false;
        }
        
        try
        {
            //remove any spaces, as they can cause an unintented exception when parsing ints
            choice.replace(" ", "");
            //get the integer value from the string
            game = Integer.parseInt(choice);
            //if they gave a weird input we'll handle it accordingly
            if(game > NUMGAMES || game <= 0)
            {
                System.out.println("You seriously only had two options and you messed that up. Try again");
                return true;
            }
        }
        //if they gave a weird input we'll handle it accordingly
        catch(NumberFormatException e)
        {
            System.out.println("Thanks for giving wrong input. Try again");
            return true;
        }
        
        //This will explain the rules to the user
        switch(game)
        {
            //number guesser
            case NUM_GUESS:
                //Run the rules out for the user.
                System.out.println("Welcome to the number guesser!\nHere's the rules:\n"
                                    + "1.Host will choose a number\n"
                                    + "2.The client will guess it\n"
                                    + "That's it\n");
                break;
            case ROCHAMBEAU:
                System.out.println("Welcome to Rock, Paper, Scissors!\n"
                                 + "Or as the cool kids call it, Rochambeau\n" 
                                 + "Simply type when prompted Rock Paper or Scissors\n"
                                 + "Remember that Rock beats scissors\n"
                                 + "Paper beats Rock\n"
                                 + "and Scissors beats paper");
            default:
                break;
        }
        //Ask if they'll host or join a game
        System.out.println("Will you host or join a game?");
        System.out.println("1)Host\n2)Join");

        //retrieve the choice to host or be the client
        choice = scan.next();
        if(isQuit(choice))
        {
            return false;
        }
        int host;
        try
        {
            //remove spaces again
            choice.replace(" ", "");
            //parse the int from the string
            host = Integer.parseInt(choice);
        }
        //if they give wrong input, we'll make them host
        catch(NumberFormatException e)
        {
            System.out.println("Thanks for giving wrong input. You're hosting.");
            host = HOST;
        }

                        //
        if(host == HOST)
        {
           //Try to set up a host
            try{
                                        //Give them some information about hosting and give them their ip address, so they can connect to someone else.
                System.out.println("Setting up host. Good Luck! Note if you'd like to stop hosting you will have to quit the program entirely. Sorry about that.");
                System.out.println("Here's your ip address. " + InetAddress.getLocalHost() + ".Give it to your friends!\nAlso you'll be on port " + (PORT + game) + " so forward that port if you're having trouble");
                //set up the host and start searching for users
                                        new Host(name, game).run();
            }
            //catch any exceptions that are thrown
            catch(Exception e)
            {
                System.out.println("You're trying to host but you're already doing that. So stop it");
                return true;
            }
        }

        //joining a game
        else
        {
            //get host info 
            System.out.println("Ok you're the client. That means you have the hard part");
            System.out.println("What's the host's ip address(Type local if on same machine");

            //get the ip from the user
            String ip = scan.next();
            if(isQuit(ip))
            {
                return false;
            }
            if(ip.equalsIgnoreCase("local"))
            {
                try
                {
                    return new Client(name, InetAddress.getLocalHost(), PORT + game, game).run();

                }
                catch(UnknownHostException e)
                {
                    System.out.println("If we got here there's a major problem. Your machine doesn't exist." + e.getMessage());
                }
                catch(IOException e)
                {
                    System.err.println("IOException found" + e.getMessage());
                }
            }
            //joining a game with a known ip address
            else
            {
                System.out.println("You gave a cool ip address there ");
                System.out.println("Attempting to connect to " + ip);
                try
                {
                    //getbyname should work if the user gives it properly in ip address format.
                    return new Client(name, InetAddress.getByName(ip), PORT + game, game).run();

                }
                //Let them know they gave a bad ip address
                catch(UnknownHostException e)
                {
                    System.out.println("I'm sorry you gave a bad ip address. Returning to game select screen");
                    return true;
                }
                catch(IOException e)
                {
                    System.out.println("IOException found while attemping to connect to a network\n" + e.getMessage());
                }   return true;

            }
        }
        
        return true;
    }
    
    private static boolean isQuit(String message)
    {
        if(message.equalsIgnoreCase("Quit"))
        {
            System.out.println("Have a nice day!");
            return true;
        }
        return false;
    }
}
