/**
 * Name:Daniel Greco
 * Filename: Message.java
 * Description: This file will pretty much be the packets that will be sent back and forth between
 * the host and client.
 */ 
import java.io.*;
public class Message implements Serializable
{
	
    private final String name;  //name of sender
    private final String message; //message they want to send
    private final int msgType; //message code
    
	//codes indicating what type of message it is
    public static final int GUESS = 1;
    public static final int STRINGMSG = 2;
    public static final int YESNO = 3;
    
    //must include name and message to send data!
    private Message()
    {
        this.name ="noname";
        this.message = null;
        msgType = 0;
    }
    
    /**
     * This constructor will be used for all creations of the message class
     * @param name is a string with the name of the sender
     * @param message is a string that is the actual message
     * @param msgType is a code indicating what type of message it is.
     */
    public Message(String name, String message, int msgType)
    {
        if(name == null)
            this.name = "badname";
        else
            this.name = name;
        if(message == null)
            this.message = "badMessage";
        else
            this.message = message;
        //depricated consider removing if safe to do 
        this.msgType = msgType;
    }
    
    /**
     * toString for Message Class. Gives just the message back.
     * @return the message string, not the class.
     */
    public String toString()
    {
        return message;
    }
    
    /**
     * isQuit will see if the message contains the key phrase Bye, which will indicate to the host and client
     * the game is done
     * 
     * @return true if the message contains the quit message, false otherwise
     * 
     */
    public boolean isQuit()
    {
        return message.equalsIgnoreCase("Bye");
    }
    
    /**
     * isWin will see if the client has won
     * @return true if they won, false otherwise.
     * 
     */
    public boolean isWin()
    {
        return message.equalsIgnoreCase("You got it!\n");
    }
    
    /**
     * isOver will be used with the rochambeau protocol. It will check to
     * see if the match is over. Note case is sensitive here since we are 
     * checking if it contains the string, not equals it.
     * 
     * @return true if the message is a win or loss scenario.
     */
    public boolean isOver()
    {
        //case sensitive!
        return message.contains("You Won!\n") || message.contains("You got destroyed!\n");
    }
    
    /**
     * isOver will be used with the rochambeau protocol. It will check to
     * see if the match is over. This version is used as a check, so we 
     * can do this for the host in the protocol as well.
     * 
     * @return true if the message is a win or loss scenario.
     */
    public static boolean isOver(String message)
    {
        return message.equalsIgnoreCase("You won!\n") || message.equalsIgnoreCase("You got destroyed!\n");
    }
    
    /**
     * getMsgType is a getter method for the variable msgType
	 *	@return msgType
     * 
     */
    public int getMsgType()
    {
        return this.msgType;
    }
    
    /**
     * getMessage is a getter method for the variable message
     * @return the string message
     */
    public String getMessage()
    {
        return this.message;
    }
    
    /**
     * getName is a getter method for the variable name
     * @return the variable name
     */
    public String getName()
    {
        return this.name;
    }
}
