/*
 * Name: Daniel Greco
 * FileName: MessageTest.java
 * Description: This class will test the functionality of class Message. Note: There
 * are only two methods that are not tested, getMsgCode and getMessage. The reason being
 * that getMessage is essentially tested in the toString test, as they both do the
 * same thing. getMsgcode is so simple that it doesn't need to be tested
 */

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
/**
 *
 * @author Daniel
 */
public class MessageTest {
    
    public MessageTest() {
    }
    
    @BeforeClass
    public static void setUpClass() {
    }
    
    @AfterClass
    public static void tearDownClass() {
    }
    
    @Before
    public void setUp() {
    }
    
    @After
    public void tearDown() {
    }

    /**
     * Test of toString method, of class Message. This won't have many tests
     * As there are only two types of input, a null object or a normal string.
     */
    @Test
    public void testToString() {
        Message msg1, msg2;
        String bad = null;
        msg1 = new Message("",bad, 5); 
        msg2 = new Message("", "Test Message", 3);
        
        //test when a null string is given
        String expResult = "badMessage";
        String result = msg1.toString();
        assertEquals(expResult, result);
        
        //test when a normal string is given
        expResult = "Test Message";
        assertEquals(expResult, msg2.toString());
        
    }

    /**
     * Test of isQuit method, of class Message. Again this will be a very
     * simply test as we're only making sure that this one statement is correct
     */
    @Test
    public void testIsQuit() {
        
        Message msg1, msg2, msg3;
        
        //Test 1: Make sure it works normally
        msg1 = new Message("Frank", "bye", 3);
        
        boolean expResult = true;
        boolean result = msg1.isQuit();
        
        assertEquals(expResult, result);
        
        //Test 2: Make sure case isn't important
        msg2 = new Message("Frank", "Bye", 3);
         expResult = true;
        result = msg2.isQuit();
        
        assertEquals(expResult, result);
        
        //Test 3:Make sure that if the word bye is contained in a message it wont
        //return true
        msg3 = new Message ("Frank", "goodbye", 3);
        expResult = false;
        result = msg3.isQuit();
        
        assertEquals(expResult, result);
        
    }

    /**
     * Test of isWin method, of class Message. Again this will be a very
     * simply test as we're only making sure that this one statement is correct
     */
    @Test
    public void testIsWin() {
        Message msg1, msg2, msg3, msg4;
        
        //Test 1: Make sure it works normally
        msg1 = new Message("Frank", "You got it!\n", 3);
        
        boolean expResult = true;
        boolean result = msg1.isWin();
        
        assertEquals(expResult, result);
        
        
        //Test 2: Make sure test null strings
        String bad = null;
        msg2 = new Message("Frank", null , 3);
         expResult = false;
        result = msg2.isWin();
        
        assertEquals(expResult, result);
        
        //Test 3:Make sure case isn't a problem
        msg3 = new Message("Frank", "YoU GoT It!\n", 4);
        expResult = true;
        result = msg3.isWin();
        
        assertEquals(expResult, result);
        
        //Test 4: Make sure no new line will return false
        msg4 = new Message("Frank", "You Got It!", 4);
        expResult = false;
        result = msg4.isWin();
        
        assertEquals(expResult, result);
    }

    /**
     * Test of getName method, of class Message. This is the only getter 
     * being tested because getMessage and the toString perform the same operation
     * and getCode is really simple and can return any result.
     */
    @Test
    public void testGetName() {
        Message msg1, msg2, msg3;
        
        //Test 1: Normal Conditions
        msg1 = new Message("daN", "Test", 3);
        
        String expResult = "daN";
        String result = msg1.getName();
        assertEquals(expResult, result);

        //Test 2: Null String Given
        String bad = null;
        msg2 = new Message(bad, "Doesn't Matter", 2);
        
        expResult = "badname";
        result = msg2.getName();
        assertEquals(expResult, result);
        
    }
    
    /**
     * Test of the two isOver methods for class Message. Remember that the 
     * message has to be case sensitive when calling it from message, but can
     * be case insensitive when using it with the static method
     */
    @Test
    public void testIsOver() {
        Message msg1, msg2, msg3;
        
        //Test 1: Wrong case
        msg1 = new Message("Dan", "You won!\n", 3);
        
        boolean expResult = false;
        boolean result = msg1.isOver();
        assertEquals(expResult, result);
        
        //Test 2: Correct winning message
        msg2 = new Message("Dan", "You Won!\n", 3);
        
        expResult = true;
        result = msg2.isOver();
        assertEquals(expResult, result);
        
        //Test 3: Correct losing message
        msg3 = new Message("Dan", "You got destroyed!\n",3 );
        
        expResult = true;
        result = msg3.isOver();
        assertEquals(expResult, result);
        
        //These two tests check the static isover method. Case can be insensitive
        assertEquals(Message.isOver("You won!\n"), true);
        assertEquals(Message.isOver("You Got destroyed!\n"), true);
    }
}
