/**
 * Name:Daniel Greco
 * Filename: NumGuessProtocol.java
 * Description: This file will handle the rules of the number guessing game 
 */ 

import java.net.*;
import java.io.*;
import java.util.Random;
import java.util.ArrayList;
import java.util.Scanner;

public class NumGuessProtocol implements Protocol {

	//state codes
    private static final int WAITING = 0;
    private static final int PLAYING = 1;
    private static final int PLAYAGAIN = 2;
    
	
    private int state;
    private String name; //host name
    private BufferedReader stdIn; //read from stdin
    private int ranNum;	//random number if the host gives a weird number
    private Random rand;
    private Scanner scan;
    private ArrayList<Integer> prevGuesses; //holds all the previous guesses, so the client can see previous guesses
    

	//we'll never get here
	private NumGuessProtocol()
	{}
	
    /**
     * Constructor for class NumGuessProtocol
	 * @param name is a string that is the name of the host
     * 
     */
    public NumGuessProtocol(String name)
    {	
        //make a new array
        this.prevGuesses = new ArrayList<Integer>();
		//set up the random number generator
        rand = new Random();
		//initialize the std in reader
        stdIn = new BufferedReader(new InputStreamReader(System.in));
		//set the state to the default state
        state = WAITING;
		//set up the scanner
        scan = new Scanner(System.in);
        this.name = name;
    }
  
    /**
     * newRand will generate a new random number, and clear the prevGuesses array.
     * Note this is no longer used.
     * 
     */
    private void newRand()
    {
        ranNum = rand.nextInt(1000) + 1; //+ 1 given because we want to go from 1-1000, not 0-999
        prevGuesses.clear();
    }
    
    /**
     * newNum will be used now because the host has to enter in a number
     * 
     */
    private void newNum()
    {
        System.out.println("Enter in a new number between 1 and 1000");

		//get the number from the user
        String ui = scan.next();
        ui.replace(" ", "");
        prevGuesses.clear();
		//read in the number and handle bad input
        try
        {
            ranNum = Integer.parseInt(ui);
			if(ranNum > 1000)
			{	
				//mod the result so we get into the range 1-1000
				ranNum = ranNum % 1000;
				++ranNum;
                                System.out.println("You gave a number that was way too high.\nModding that number by 1000 gives " + ranNum);
			}
			//negative or zero
			else if(ranNum < 1)
			{
				//flip the sign if it's less than zero
				ranNum *= -1;
				//mod it if it's too high
				ranNum = ranNum % 1000;
				//add one so we're in the range
				++ranNum;
                                System.out.println("You gave a number that was way too low.\nModding that number by 1000 gives " + ranNum);
			}
        
        }
		//handle when they give bad date that's not a number
        catch(NumberFormatException e)
        {
            System.out.println("So you gave something that wasn't a number, well here's the fun part, i'm picking it now. Enjoy having no part in the game.");
            newRand();
        }
        
    }
    
    
    /**
     * getMessage will use the guess and generate a message that will be sent to the client
     * @param guess is an int that contains the current guess the client made.
	 * @return the message that will be sent back to thec lient
     */
    private String getMessage(int guess)
    {
        //add to previous guesses 
        prevGuesses.add(guess);
        if(guess > ranNum)
        {
            return "Your guess was too high\n";
        }
        else if(guess < ranNum)
        {
            return "Your guess was too low\n";
        }
        else if(guess > 1000 || guess < 1)
        {
            return "You should remember that it's between 1-1000\n"; 
        }
        else
        {
            return "You got it!\n";
        }
        
    }
    
    /**
     * processInput is the main method of this class. It'll handle the input the client sent and generate the output to give back to them.
	 * @param theInput is a message object that contains the message the user gave
     * @return the host message back to the client
     */
    public Message processInput(Message theInput) throws IOException
    {
        String theOutput = null;
        int theMsg = Message.STRINGMSG; //always a string
        String clientName; 
        int guess;
		//see if we have a clientName this time. If we do store it.
        try
        {
            clientName = theInput.getName();
        }
        catch(Exception e)
        {
            clientName = "the client";
        }
	
        //if we're processing input in this area, that means we're now connected
        switch(state) 
        {
			//default state!
            case WAITING: 
            {
                //Step 4: Send message to client because we're now connected
                theOutput = "Connection to " + name + " established\nGuess a number between 1- 1000";
                System.out.println("Player Connected");
				//change state to playing as we now have a connection
                state = PLAYING;
				//get a new random number
                newNum();
                theMsg = Message.STRINGMSG;
                break;
            }
        
            case PLAYING:
            {
                //check if the player quitted
                if(theInput.isQuit())
                {
					//go back to waiting and sent the quit message back. So they know to leave
                    state = WAITING;
                    System.out.println(theInput);
                    theOutput = "Bye";
                }   
                else
                {
                    //Step 10: Print message on screen
                    System.out.println(clientName +  " guessed " + theInput);
                    //Step 11:Read Message from user
                    if(theInput.getMsgType() == Message.GUESS)
                    {   
                        String message = theInput.getMessage();
                        message = message.replace(" ", "");
						//parse their int
                        try
                        {
                            guess = Integer.parseInt(message);
                        }
						//if the user gives bogus input we'll return a message here yelling at them.
                        catch(NumberFormatException e)
                        {
                            return new Message(name, "Hey you gave a weird input, try again please",  theMsg);
                        }
						
						//start to build the normal string to send back
                        String guessOutput = getMessage(guess);
						//if they got to this then they won
                        if(guessOutput.equals("You got it!\n"))
                        {
							//inform the host the game ended.
                            theOutput = guessOutput;
                            state = PLAYAGAIN;
                            System.out.println("If you'd like to leave feel free to do so. The client is about to choose if we're playing another round");
                            return new Message(name, theOutput, theMsg);
                        }
						//add the output to the return string
                        theOutput = guessOutput;
						//give the player a string representation of their array
                        theOutput += prevGuesses.toString();
						//also let them know how many guesses they've taken already.
                        theOutput += "\nYou've taken " + prevGuesses.size() + " guesses so far. \n"; 
                        
                        
                    }
                }
            }
                break;
			//this will be reached when the client successfully guesses the number
            case PLAYAGAIN:
            {
				//get the message
                String message = theInput.getMessage();
					//if they don't give yes, they're quitting
                    if(message.equalsIgnoreCase("Yes"))
                    {
						//get a new number
                        state = PLAYING;
                        theOutput = "To quote coolio \"Aw Here It Goes\". Try to guess again!";
                        newNum();
                    }
					
                    else
                    {
						//go back to waiting
                        state = WAITING;
                        System.out.println(theInput);
						//send a quit message to the client, so they can disconnect.
                        theOutput = "Bye";    
                    } 
                break;
            }
            default:
                //throw an exception or something because we should never get here
                break;
        }
		//return the message if we haven't already
        return new Message(name, theOutput, theMsg);
    } //processinput
}